;
/*
Copyright 2009, Sebastian Hennebrueder, http://www.laliluna.de

Licensed under the Open Source Robin Hood License, Version 0.1 (the "License");
you may not use this file except in compliance with the License. You may obtain a copy of the License at
http://www.laliluna.de/open-source-robin-hood-license.html

Unless required by applicable law or agreed to in writing, software distributed under the License is
distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, without
even the implied warranty of FITNESS FOR A PARTICULAR PURPOSE.

See the License for the specific language governing permissions and limitations under the License.

Richness criteria:
For private use: your income is above 50000 EUR before tax per year.
For organisation: your total turnover is above 1 Million EUR per year.
 */
(function($) {


	/**
	 *
	 * @param url
	 * @param options
	 * max - max entries which can be selected, -1 for unlimited
	 * minChars - min characters before data is loaded
	 * loadData - allows to overwrite default data loader, see loadData function
	 * buildSelected - allows to overwrite how the li element for selected entries is build -> see buildSelected function
	 * updateHidden ->allows to overwrite how hidden values are updated, see function updateHidden
	 * buildSelectLi - allows to overwrite the calculation of the scroll position in the selection div
	 * timerDelay - the key event handler waits for a moment to let the key event pass through. Otherwise we don't get
	 * the last pressed character. If you run into problems, you may change this value
	 * useCache - allow to cache values
	 * matchAny - if true, defines if cached values are filtered using a contains (=any) match instead of start wirth.
	 */
	$.fn.autocomplete = function(url, options) {

		// define defaults
		options = $.extend({max: 1}, options)
		this.each(function() {
			new $.AutoComplete(this, url, options);
		});

	}

	$.AutoComplete = function(input, url, options) {
		var _url = url;
		var _loadData = (options.loadData ? options.loadData : loadData);
		var _input = $(input);
		// used to wait for key events to finish
		var timerId;
		var timerDelay = (options.timerDelay ? options.timerDelay : 20);

		var ul = _input.parent().parent();
		var hiddenInputId = '#' + _input.attr('id') + '-internal';

		var _selected = new $.AutoComplete.Selected(ul, $(hiddenInputId), options);
		var dataArea = $('<div class="autocompletedata"/>');
		var divId = _input.attr('id') + '-selection';
		dataArea.attr('id', divId);

		ul.after(dataArea);
		var _select = new $.AutoComplete.Select(_selected, divId, options);
		// caching in case of backspace
		var lastUserInput = null;

		// prevent form submit for opera, if return is used to select value
		$.browser.opera && $(input.form).bind("submit.autocomplete", function() {
			return false;
		});

		function prepareDelayedLoad() {
			var userInput = _input.val();
			if (!userInput)
				userInput = '';
			else
				userInput = $.trim(userInput);

			if ((options.minChars > 0) && (userInput.length < options.minChars))
				_select.displayData({});
			else {
				if (options.useCache && lastUserInput && $.startsWith(userInput, lastUserInput)) {
					_select.displayCached(userInput);
				}
				else {
					lastUserInput = (userInput.length > 0 ? userInput : null);
					_loadData(userInput, _select.displayData);
				}
			}
		}


		_input.bind($.browser.mozilla ? "keypress" : "keydown", function(event) {

			var pressedKey = event.keyCode;
			window.status = 'key' + pressedKey;
			switch (pressedKey) {
				case Keys.ESC:
					event.preventDefault();
					_select.hide();
					break;
				case Keys.RETURN:
					if (_select.isVisible()) {
						event.preventDefault();
						_selected.add(_select.current());
						_select.hide();
					}
					break;
				case Keys.UP:
					event.preventDefault();
					_select.previous();
					break;
				case Keys.DOWN:
					event.preventDefault();
					_select.next();
					break;
				case Keys.PAGEUP:
					event.preventDefault();
					_select.previousPage();
					break;
				case Keys.PAGEDOWN:
					event.preventDefault();
					_select.nextPage();
					break;
				case Keys.TAB:
					_select.hide();
					break;
				case Keys.DEL:
					if ($.trim(_input.val()).length > 0) {
						clearTimeout(timerId);
						timerId = setTimeout(prepareDelayedLoad, timerDelay);
					}
					break;
				case Keys.BACKSPACE:
					if ($.trim(_input.val()).length == 0) {
						_selected.removeLast();
					}
					else {
						clearTimeout(timerId);
						timerId = setTimeout(prepareDelayedLoad, timerDelay);
					}
					break;
				default:
					clearTimeout(timerId);
					timerId = setTimeout(prepareDelayedLoad, timerDelay);
			}
		});

		/**
		 * Loads the data and builds an array of objects having a label and a value.
		 * Can be replaced by a custom function. This functions makes a request to load JSOn data
		 * @param userInput - partial input of the user
		 */
		function loadData(userInput, displayFunction) {
			$.getJSON(_url, {'autocompleteinput': userInput}, displayFunction);
		}

	};

	/**
	 * Deals with selected items
	 * @param ul - the ul element holding selected items as jquery selection
	 * @param hidden - the hidden input element holding the value as jquery selection
	 * @param options - an object structure holding options
	 */
	$.AutoComplete.Selected = function(ul, hidden, options) {
		var _max = options.max;
		var _buildSelected = (options.buildSelected ? options.buildSelected : buildSelected);
		var _updateHidden = (options.updateHidden ? options.updateHidden : updateHidden)

		/**
		 * Removes the last selected entry
		 */
		function removeLast() {
			ul.find('li.autocomplete_selected:last').remove();
			_updateHidden();
		}

		/**
		 * Default method to deal with the hidden value field. Can be overwritten with options
		 */
		function updateHidden() {
			var selected = ul.find('li.autocomplete_selected');
			var values = $.map(selected, function(value) {
				return $(value).attr('autovalue');
			});
			hidden.val(values.join(','));
		}

		/**
		 * Builds an li element holding a selected value. Can be overwritten using options
		 * @param entry
		 */
		function buildSelected(entry) {
			var li = $('<li class="autocomplete_selected"/>');
			li.text(entry.label);
			li.attr('autovalue', entry.value);
			li.append($('<a href="#"/>'));
			return li;
		}

		/* public API */
		return{
			/**
			 * Adds an entry to the select list
			 * @param entry - an object having a label and a value property
			 */
			add:function(entry) {
				// -1 because of the input field which doesn't count
				if (_max != -1 && (ul.children().size() - 1) >= _max)
					removeLast();

				// insert right before the input li
				ul.find('li.autocomplete_input').before(buildSelected(entry));

				$('li.autocomplete_selected > a').click(function(e) {
					$(this).parent().remove();
					_updateHidden();
				});

				_updateHidden();

				ul.find('li > input').val('').focus();
			},
			/**
			 * Removes the last selected entry
			 */
			removeLast: removeLast,
			/**
			 * Returns the bottom position of the selection ul.	Used by the select div to position itself.
			 * positioned
			 */
			bottom: function() {
				var offset = ul.offset();
				return offset.top + ul.height() + 'px';
			},
			/**
			 * Returns the left position of the selection ul. Used by the select div to position itself.
			 */
			left: function() {
				return ul.offset().left;
			}
		}
	}

	/**
	 * Deals with the select window
	 * @param selected - an instance of $.Autocomplete.Selected which holds the selected value
	 * @param divId - the id of the div inside which the selected roles are shown
	 */
	$.AutoComplete.Select = function(selected, divId, options) {
		var _data = [];
		var _cached = null;
		var _current;
		var _selected = selected;
		var _divId = '#' + divId;
		var _div = $(_divId);
		var _buildSelectLi = (options.buildSelectLi ? options.buildSelectLi : buildSelectLi);
		var _scrollPosition = (options.scrollPosition ? options.scrollPosition : scrollPosition);
		/**
		 * Repaints the div
		 */
		function repaint() {
			if (_data.empty) {
				_div.empty();
				return;
			}
			// reposition the div
			_div.css({top: _selected.bottom(),left:_selected.left()});

			var ul = $('<ul/>');
			$.each(_data, function(n) {
				var li = _buildSelectLi(this, n);
				ul.append(li);
			});
			_div.empty();
			_div.append(ul);
			$(_divId + ' > ul > li').click(function(e) {
				self = $(this);
				_selected.add({label: self.text(), value: self.attr('autovalue')})
				_div.hide();
			});
		}

		/**
		 * Creates an li element
		 * @param entry - an object having a label and a value property
		 * @param n - the index position of the element to allow even/odd row styles
		 */
		function buildSelectLi(entry, n) {
			var li = $('<li/>');
			li.text(entry.label);
			if (n == _current)
				li.attr('class', 'autocomplete_current');
			else
				li.attr('class', (n % 2 == 0 ? 'autocomplete_even' : 'autocomplete_odd'));
			li.attr('autovalue', entry.value);
			return li;
		}

		/**
		 * Calculates the scroll position to keep the current entry in the middle.
		 * Can be overwritten using options
		 */
		function scrollPosition() {
			// work around: reusing li doesn't work. Offset fails with an exception
			var delta = $(_divId + ' li.autocomplete_current').offset().top;
			var offSet = _div.find('ul:first-child').offset().top;
			return delta - offSet - _div.height() / 3;
		}

		/**
		 * Navigates the selection
		 * @param step
		 */
		function navigate(step) {
			if (_div.is(':hidden'))
				_div.show();

			var li = _div.find('ul').children().get(_current);
			$(li).attr('class', (_current % 2 == 0 ? 'autocomplete_even' : 'autocomplete_odd'));
			_current += step;
			if (_current >= _data.length)
				_current = (_data.length - 1);
			else if (_current < 0)
				_current = (_data.length > 0 ? 0 : - 1);

			var li = $(_div.find('ul').children().get(_current));
			li.attr('class', 'autocomplete_current');

			_div.attr('scrollTop', _scrollPosition);
		}

		/* public API */
		return {
			isVisible: function() {
				return !(_div.is(':hidden'));
			},
			show: function() {
				_div.show();
			},

			hide: function() {
				_div.hide();
			},

			next: function() {
				navigate(1);
			},

			previous: function() {
				navigate(-1);
			},
			nextPage: function() {
				navigate(4);
			},

			previousPage: function() {
				navigate(-4);
			},

			/**
			 * Returns an object with label and value holding the current selected entry
			 */
			current: function() {
				var li = _div.find('ul').children().get(_current);
				if (li) {
					li = $(li);
					return {label: li.text(), value: li.attr('autovalue')};
				}
				else
					return null;
			},

			/**
			 * Builds the select div, parses the data and shows it
			 * @param data - a JSON array holding objects with label and value
			 */
			displayData: function(data) {
				_cached = null;
				_data = data;
				_current = _data.length > 0 ? 0 : -1;
				repaint();
				_div.show();
			},

			/**
			 * Removes not matching data
			 * @param userinput
			 */
			displayCached: function(userinput) {
				if (!_cached) {
					_cached = _data;
				}
				_data = []
				if (options.matchAny) {
					$.each(_cached, function(n) {
						if ($.AutoComplete.contains(this.label, userinput))
							_data[_data.length] = this;
					});
				} else {
					$.each(_cached, function(n) {
						if ($.AutoComplete.startsWith(this.label, userinput))
							_data[_data.length] = this;
					});
				}
				_current = _data.length > 0 ? 0 : -1;
				repaint();
				_div.show();
			}
		}

	};
	var Keys = {
		UP: 38,
		DOWN: 40,
		DEL: 46,
		TAB: 9,
		RETURN: 13,
		ESC: 27,
		COMMA: 188,
		PAGEUP: 33,
		PAGEDOWN: 34,
		BACKSPACE: 8
	};

	/**
	 * Validates if <i>str</> contains with the String <i>mtch</i>.
	 * @param str
	 * @param start
	 */
	$.AutoComplete.contains = function (str, part) {
		return (str.indexOf(part) != -1);
	};
	/**
	 * Validates if <i>str</> starts with the String <i>mtch</i>.
	 * @param str
	 * @param start
	 */
	$.AutoComplete.startsWith = function (str, start) {
		var string = String(str);
		var length = String(start).length;

		return (string.substring(0, length) == start);
	};
})(jQuery);